@@ -508,7 +508,38 @@ static const char *am_set_setenv_slot(cmd_parms *cmd,
                                       const char *oldName)
 {
     am_dir_cfg_rec *d = (am_dir_cfg_rec *)struct_ptr;
-    apr_hash_set(d->envattr, oldName, APR_HASH_KEY_STRING, newName);
+    /* Configure as prefixed attribute name */
+    am_envattr_conf_t *envattr_conf = (am_envattr_conf_t *)apr_palloc(cmd->pool, sizeof(am_envattr_conf_t));
+    envattr_conf->name = newName;
+    envattr_conf->prefixed = 1;
+    apr_hash_set(d->envattr, oldName, APR_HASH_KEY_STRING, envattr_conf);
+    return NULL;
+}
+
+/* This function handles the MellonSetEnvNoPrefix configuration directive.
+ * This directive allows the user to change the name of attributes without prefixing them with MELLON_.
+ *
+ * Parameters:
+ *  cmd_parms *cmd       The command structure for the MellonSetEnv
+ *                       configuration directive.
+ *  void *struct_ptr     Pointer to the current directory configuration.
+ *  const char *newName  The new name of the attribute.
+ *  const char *oldName  The old name of the attribute.
+ *
+ * Returns:
+ *  This function will always return NULL.
+ */
+static const char *am_set_setenv_no_prefix_slot(cmd_parms *cmd,
+                                      void *struct_ptr,
+                                      const char *newName,
+                                      const char *oldName)
+{
+    am_dir_cfg_rec *d = (am_dir_cfg_rec *)struct_ptr;
+    /* Configure as not prefixed attribute name */
+    am_envattr_conf_t *envattr_conf = (am_envattr_conf_t *)apr_palloc(cmd->pool, sizeof(am_envattr_conf_t));
+    envattr_conf->name = newName;
+    envattr_conf->prefixed = 0;
+    apr_hash_set(d->envattr, oldName, APR_HASH_KEY_STRING, envattr_conf);
     return NULL;
 }
 
@@ -951,22 +982,30 @@ const command_rec auth_mellon_commands[] = {
         am_set_setenv_slot,
         NULL,
         OR_AUTHCFG,
-        "Renames attributes received from the server. The format is"
+        "Renames attributes received from the server while retaining prefix MELLON_. The format is"
         " MellonSetEnv <old name> <new name>."
         ),
+     AP_INIT_TAKE2(
+        "MellonSetEnvNoPrefix",
+        am_set_setenv_no_prefix_slot,
+        NULL,
+        OR_AUTHCFG,
+        "Renames attributes received from the server without adding prefix. The format is"
+        " MellonSetEnvNoPrefix <old name> <new name>."
+        ),
     AP_INIT_FLAG(
         "MellonSessionDump",
         ap_set_flag_slot,
         (void *)APR_OFFSETOF(am_dir_cfg_rec, dump_session),
         OR_AUTHCFG,
-        "Dump session in environement. Default is off"
+        "Dump session in environment. Default is off"
         ),
     AP_INIT_FLAG(
         "MellonSamlResponseDump",
         ap_set_flag_slot,
         (void *)APR_OFFSETOF(am_dir_cfg_rec, dump_saml_response),
         OR_AUTHCFG,
-        "Dump SAML authentication response in environement. Default is off"
+        "Dump SAML authentication response in environment. Default is off"
         ),
     AP_INIT_RAW_ARGS(
         "MellonRequire",
@@ -1064,6 +1103,13 @@ const command_rec auth_mellon_commands[] = {
         OR_AUTHCFG,
         "List of IdP entityId to ignore."
         ),
+    AP_INIT_TAKE1(
+        "MellonSPentityId",
+        ap_set_string_slot,
+        (void *)APR_OFFSETOF(am_dir_cfg_rec, sp_entity_id),
+        OR_AUTHCFG,
+        "SP entity Id to be used for metadata auto generation."
+        ),
     AP_INIT_TAKE12(
         "MellonOrganizationName",
         am_set_langstring_slot,
@@ -1231,6 +1277,7 @@ void *auth_mellon_dir_config(apr_pool_t *p, char *d)
     dir->probe_discovery_timeout = -1; /* -1 means no probe discovery */
     dir->probe_discovery_idp = apr_table_make(p, 0);
 
+    dir->sp_entity_id = NULL;
     dir->sp_org_name = apr_hash_make(p);
     dir->sp_org_display_name = apr_hash_make(p);
     dir->sp_org_url = apr_hash_make(p);
@@ -1400,6 +1447,10 @@ void *auth_mellon_dir_merge(apr_pool_t *p, void *base, void *add)
                           add_cfg->idp_ignore :
                           base_cfg->idp_ignore;
 
+    new_cfg->sp_entity_id = (add_cfg->sp_entity_id ?
+                             add_cfg->sp_entity_id :
+                             base_cfg->sp_entity_id);
+
     new_cfg->sp_org_name = apr_hash_copy(p,
                           (apr_hash_count(add_cfg->sp_org_name) > 0) ?
                            add_cfg->sp_org_name : 
